﻿local FS = LibStub("AceAddon-3.0"):NewAddon("FaceShooter", "AceConsole-3.0", "AceEvent-3.0")

-- defaults used if FS is started for the first time on a given char
FS.defaults = {
	profile = {
		lock = false,
		showInRaid = true,
		showInParty = true,
		showWhenSolo = true,
		maxWait = 200,
		damping = 2,
		["debug"] = {
			point = "CENTER",
			x = 0,
			y = 0,
		},
		["ba"] = {
			suggestBA = "always",
			saveFocusBA = true,
		},
		["db"] = {
			suggestDB = "always",
			minTTL = 15,
		},
		["AMoC"] = {
			suggestAMoC = "always",
			minTTL = 15,
		},
		["GT"] = {
			suggestGT = "always",
		},
		["PS"] = {
			suggestPS = "always",
		},
		["Brg"] = {
			suggestBrg = "always",
		},
		["CA"] = {
			suggestCA = "always",
		},
		["ET"] = {
			suggestET = "never",
			minTTL = 20,
		},
		["button1"] = {
			enabled = true,
			point = "CENTER",
			x = -100,
			y = 0,
			scale = 100,
			alpha = 100,
		},
		["button2"] = {
			enabled = true,
			wasEnabled = true,
			point = "CENTER",
			x = -50,
			y = 0,
			scale = 100,
			alpha = 100,
		},
	},
}

-- initializes FS - called when the addon is loaded
function FS:OnInitialize()
	-- don't load FS unless player is at least level 90 and hunter
	local _, playerClass = UnitClass("player")
	local unitLevel = UnitLevel("player")
	if playerClass ~= "HUNTER" or unitLevel < 90 then
		self:Print("FaceShooter: Only Hunters of level 90+ are supported. Deactivating Add-on...")
		self:Disable()
		return false
	end


	-- read the savedvariables
	self.db = LibStub("AceDB-3.0"):New("FSDB2", FS.defaults)

	-- ACE3 style options table
	FS.options = {
		type = "group",
		name = "FaceShooter",
		handler = FS,
		args = {
			debug = {
				type = "toggle",
				name = "debug",
				get = "IsDebug",
				set = "ToggleDebug",
				hidden = true,
			},
			log = {
				type = "toggle",
				name = "log",
				get = "IsLog",
				set = "ToggleLog",
				hidden = true,
			},
			lock = {
				type = "toggle",
				name = "Lock frames",
				get = "IsLock",
				set = "ToggleLock",
				width = "full",
				order = 1,
			},
			showInRaid = {
				type = "toggle",
				name = "Show in Raid",
				get = "IsShowInRaid",
				set = "ToggleShowInRaid",
				order = 2,
			},
			showInParty = {
				type = "toggle",
				name = "Show in party",
				get = "IsShowInParty",
				set = "ToggleShowInParty",
				order = 3,
			},
			showWhenSolo = {
				type = "toggle",
				name = "Show when solo",
				get = "IsShowWhenSolo",
				set = "ToggleShowWhenSolo",
				order = 4,
			},
			maxWait = {
				type = "range",
				name = "Max. time to wait for signature shots (in ms)",
				min = 0,
				max = 1000,
				step = 100,
				width = "double",
				get = "GetMaxWait",
				set = "SetMaxWait",
				width = "full",
				order = 5,
			},
			ba = {
				type = "group",
				name = "Black Arrow Options",
				order = 6,
				args = {
					suggestBA = {
						type = "select",
						name = "Suggest Black Arrow",
						values = {
							never = "Never",
							level = "Bosslevel",
							always = "Always",
						},
						get = "GetSuggestBA",
						set = "SetSuggestBA",
						order = 1,
					},
					saveFocusBA = {
						type = "toggle",
						name = "Save focus for sig. shot when suggesting Black Arrow",
						get = "IsSaveFocusBA",
						set = "ToggleSaveFocusBA",
						width = "full",
						order = 2
					},
				},
			},
			ET = {
				type = "group",
				name = "Explosive Trap Options",
				order = 7,
				args = {
					suggestET = {
						type = "select",
						name = "Suggest Explosive Trap",
						values = {
							never = "Never",
							level = "Bosslevel",
							always = "Always",
						},
						get = "GetSuggestET",
						set = "SetSuggestET",
						order = 1,
					},
					ET_TTL = {
						type = "range",
						name = "Min Time to Live to use Explosive Trap",
						min = 0,
						max = 30,
						step = 1,
						width = "double",
						get = "GetETTTL",
						set = "SetETTTL",
						width = "full",
						order = 2,
						},
					},
				},
			dbopt={
				type = "group",
				name = "Dire Beast Options",
				order = 8,
				args = {
					db = {
						type = "select",
						name = "Suggest Dire Beast",
						values = {
							never = "Never",
							level = "Bosslevel",
							always = "Always",
							},
						get = "GetSuggestDB",
						set = "SetSuggestDB",
						order = 7,
					},
					dbttl = {
						type = "range",
						name = "Min Time to Live to use Dire Beast",
						min = 0,
						max = 20,
						step = 1,
						width = "double",
						get = "GetDbTTL",
						set = "SetDbTTL",
						width = "full",
						order = 5,
						},
					},
				},
			amocopt={
				type = "group",
				name = "A Murder of Crows Options",
				order = 9,
				args = {
					amoc = {
					type = "select",
					name = "Suggest A Murder of Crows",
					values = {
						never = "Never",
						level = "Bosslevel",
						always = "Always",
						},
					get = "GetSuggestAMoC",
					set = "SetSuggestAMoC",
					order = 8,
				},
				amocttl = {
					type = "range",
					name = "Min Time to Live to use AMoC",
					min = 0,
					max = 20,
					step = 1,
					width = "double",
					get = "GetAMoCTTL",
					set = "SetAMoCTTL",
					width = "full",
					order = 5,
					},
				},
			},
			GT = {
				type = "select",
				name = "Suggest Glaive Toss",
				values = {
					never = "Never",
					level = "Bosslevel",
					always = "Always",
					},
				get = "GetSuggestGT",
				set = "SetSuggestGT",
				order = 10,
				},
			CA = {
				type = "select",
				name = "Prioritize for Careful Aim",
				values = {
					never = "Never",
					level = "Bosslevel",
					always = "Always",
					},
				get = "GetSuggestCA",
				set = "SetSuggestCA",
				order = 11,
				},
			PS = {
				type = "select",
				name = "Suggest Power Shot",
				values = {
					never = "Never",
					level = "Bosslevel",
					always = "Always",
					},
				get = "GetSuggestPS",
				set = "SetSuggestPS",
				order = 12,
				},
			Brg = {
				type = "select",
				name = "Suggest Barrage",
				values = {
					never = "Never",
					level = "Bosslevel",
					always = "Always",
					},
				get = "GetSuggestBrg",
				set = "SetSuggestBrg",
				order = 13,
				},
			},
		}

	-- extend the options table to the necessary amount of shots
	for i = 1, 2 do
		FS.options.args["button"..i] = {
			type = "group",
			name = "Shot recom. Frame "..i,
			args = {
				enabled = {
					type = "toggle",
					name = "Enable frame",
					get = "IsEnabled",
					set = "ToggleEnabled",
					width = "full",
					order = 1,
				},
				scale = {
					type = "range",
					name = "Frame scale",
					min = 50,
					max = 200,
					bigStep = 10,
					get = "GetScale",
					set = "SetScale",
					order = 2,
				},
				alpha = {
					type = "range",
					name = "Frame alpha",
					min = 10,
					max = 100,
					bigStep = 5,
					get = "GetAlpha",
					set = "SetAlpha",
					order = 3,
				},
			},
		}
		if i == 1 then FS.options.args.button1.args.enabled = nil
		else
			FS.options.args["button"..i].args.scale.disabled = not self.db.profile["button"..i]["enabled"]
			FS.options.args["button"..i].args.alpha.disabled = not self.db.profile["button"..i]["enabled"]
		end		
	end

	-- disable options as required
	self:CheckForDisabled()
	
	-- setup the options table based on the options table
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FaceShooter", FS.options,"/fs","/face")
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FaceShooter", "FaceShooter")
	self:RegisterChatCommand("fs", "ChatCommand")
	self:RegisterChatCommand("faceshooter", "ChatCommand")
end

-- starts FS up - called when the addon is enabled
function FS:OnEnable()

	-- don't load FS unless player is at least level 90 and hunter
	local _, playerClass = UnitClass("player")
	local unitLevel = UnitLevel("player")
	if playerClass ~= "HUNTER" or unitLevel < 90 then
		self:Print("FaceShooter: Only Hunters of level 90+ are supported. You should deactivate the add-on on this character.")
		self:Disable()
		return false
	end
	
	-- some FS global vars
	self.version = "1.3.0"
	self.state = "idle"
	self.tslu = 0
	self.ooc = not UnitAffectingCombat("player")
	self.level = UnitLevel("player")
	self.guid = UnitGUID("player")
	self.gcd = 0
	self.avrgLatency = 0
	self.debug = false
	self.spellNum = {}
	self.spellName = {}
	self.spellSentAt = {}
	self.lastCastSentAt = 0
	self.spellStartedAt = {}
	self.lastCastStartedAt = 0
	self.lastCastSucceededAt = 0
	self.spellEndedAt = {}
	self.freeTables = {}
	self.ksslot = 0
	self.perks = UnitLevel("player") >= 98
	--self.perks = true --for testing
	
	-- the polling intervals for the different states
	self.intervals = {
		["idle"] = 1,
		["castSent"] = 0.5,
		["castStarted"] = 0.1,
		["succeeded"] = 0.1,
		["latency"] = 0.5,
		["waiting"] = 0.5,
	}

	-- spellIDs of used spells and talents
	self.spellID = {
		-- Shots and spells
		["SS"] = 56641, -- Steady Shot
		["CoS"] = 77767, -- Cobra Shot
		["CS"] = 53209, -- Chimaera Shot
		["BA"] = 3674, -- Black Arrow
		["ES"] = 53301, -- Explosive Shot
		["AimS"] = 19434, -- Aimed Shot
		["AS"] = 3044, -- Arcane Shot
		["KS"] = 53351, -- Kill Shot
		["KC"] = 34026, -- Kill Command
		["ET"] = 13813, -- Explosive Trap
		["MS"] = 2643, -- Multi-Shot
		["TQ"] = 19801, -- Tranquillizing Shot
		["DB"] = 120679, -- Dire Beast
		["AMoC"] = 131894, -- A Murder of Crows
		["GT"] = 117050, -- Glaive Toss
		["PS"] = 109259, -- Power Shot
		["BRG"] = 120360, -- Barrage
		["FS"] = 152245, -- Focusing Shot
		-- Off GCD spells
		["AutoS"] = 75, -- Auto Shot
		["TL"] = 77769, -- Trap Launcher
		["RF"] = 3045, -- Rapid Fire
		-- Buffs
		["LnL"] = 168980, -- Lock and Load
		["TotH"] = 34720, -- Thrill of the Hunt
		["SF"] = 177668, -- Steady Focus
		["FZ"] = 19615, -- Frenzy
		["BW"] = 19574, -- Bestial Wrath
		["RF"] = 3045, -- Rapid Fire
		-- Debuffs
		["SpS"] = 118253, -- Serpent Sting
		-- Others
		["TT"] = 165803, -- Telaari Talbuk (Allows mounted combat)
		["FWW"] = 164222, -- Frostwolf War Wolf (Allows mounted combat)
	}
	-- create the base resources object
	self.resources = self:CreateResources()
	
	-- register for needed events
	self:RegisterEvent("PLAYER_ALIVE")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_FACTION")
	
	self.frame = CreateFrame("Frame")
	self.frame:SetScript("OnUpdate", function(this, elapsed) self:OnUpdate(elapsed) end)
	
	-- setup the button frames
	for i = 1, 2 do
		if self.db.profile["button"..i]["enabled"] then self["button"..i] = self:CreateButton("button"..i) end
	end

end

-- disables FS
function FS:OnDisable()
    -- Called when the addon is disabled
	self:UnregisterEvent("PLAYER_TALENT_UPDATE")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_FACTION")
	for name, module in FS:IterateModules() do
		module:DisableModule()
	end
end

local timeStamp = 0
-- basetime for FS that calls the scheduler at given intervals
function FS:OnUpdate(elapsed)
	self.tslu = self.tslu + elapsed
	if self.tslu > self.intervals[self.state] then
		self:Scheduler()
	end
end

-- schedules the different tasks of FS
function FS:Scheduler()
	local now, i = GetTime()
	-- set time since last update to 0 (it's being updated now!)
	self.tslu = 0
	
	-- update the debugframe
	if self.debug then
		self.debugFrm:GetDebugStr()
		self.debugFrm:WriteText()
	end

	-- set to idle state when mounting
	if self:IsMounted() and not self.isMounted then
		self.isMounted = true
		self:LooseTarget()
	-- set to active state when unmounting
	elseif not self:IsMounted() and self.isMounted then
		self.isMounted = false
		if self:IsLegitTarget() then self:AcquireTarget() end
	end
	
	-- break if in idle state
	if self.state == "idle" then
		return nil
	end

	-- update the debugframe
	if self.debug then
		if self.lastState ~= self.state then
			self.lastState = self.state
		end
		self.debugFrm:GetDebugStr()
		self.debugFrm:WriteText()
		if self.log then self:WriteLogEntry("Before") end
	end
	
	-- check for interrupts
	self:CheckForInterrups()
	
	-- check if gcd is up and set state accordingly
	if self.state == "succeeded" and self.gcd <= now then
		self.state = "waiting"
		if self.spellName[1] then --push a new empty spell slot in the queue 
			table.insert(self.spellName, 1, nil)
			table.remove(self.spellName, 5)
			table.insert(self.spellSentAt, 1, nil)
			table.remove(self.spellSentAt, 5)
			table.insert(self.spellStartedAt, 1, nil)
			table.remove(self.spellStartedAt, 5)
			table.insert(self.spellEndedAt, 1, nil)
			table.remove(self.spellEndedAt, 5)
		end
	end

	-- in castSent state get current resources, calculate resulting resources if that spell is cast
	if self.state == "castSent" then -- we are in the middle of a cast, i.e. not an instant shot
		self.resources:GetResources()
		self.resources:CastSpell("fs")
		self.button1.resources:SetResources(self.resources)
		self.button1.resources:FocusGains()
		--self.button1:DrawCDF(self:GetGCD())
	-- in succeeded and castStarted state most resources should be accurate according to the api
	elseif self.state == "succeeded" or self.state == "castStarted" then
		self.resources:GetResources() -- Get resources as they are now
		self.resources:CastSpell("fs") -- Modify those resources as though we have cast the current Spell
		self.button1.resources:SetResources(self.resources) -- Pass the modified resources to button1.resources
		self.button1.resources:FocusGains() -- Modify button1.resources if the cast spell increases focus (SS or CoS)
	-- in waiting state we can use current resources unmodified
	elseif self.state == "waiting" then --We don't have to modify resources with CastSpell
		self.resources:GetResources()
		self.button1.resources:SetResources(self.resources)
		self.button1.resources:FocusGains()
	end
	

	-- pick the most suitable spell with those resources
	if self.state ~= "latency" then
		for i = 1, 2 do
			local thisButton, nextButton = "button"..i, "button"..(i + 1)
			self[thisButton].resources:PickSpell(thisButton)
			if i < 2 and self.db.profile[nextButton].enabled then
				--pass the button1.resources to button2.resources
				self[nextButton].resources:SetResources(self[thisButton].resources)
				self[nextButton].resources:CastSpell(nextButton)
				self[nextButton].resources:FocusGains()
			else
				break
			end
		end
	end

	-- update the debugframe
	if self.debug then
		if self.lastState ~= self.state then
			self.lastState = self.state
		end
		self.debugFrm:GetDebugStr()
		self.debugFrm:WriteText()
		if self.log then self:WriteLogEntry("After") end
	end
	
	-- after cast has been sent and scheduler is done change to latency state
	if self.state == "castSent" then self.state = "latency" end	
	
end
