local FS = LibStub("AceAddon-3.0"):GetAddon("FaceShooter")

-- fires once the client is loaded
function FS:PLAYER_ALIVE()
	self.spec = self:GetTalents()
	self.resources:GetBuffs()
	self.resources:GetSpellCosts()
	
	-- Find where Kill Shot KS is placed on the actionbars
	if FS.spec == "BM" or FS.spec == "MM" then
		self:RegisterEvent("ACTIONBAR_SLOT_CHANGED") -- For Kill Shot CD
		self:FindKillShotSlot()
	end
	
	self:UnregisterEvent("PLAYER_ALIVE")
end

-- fires whenever at the loading screen
function FS:PLAYER_ENTERING_WORLD()
	self.spec = self:GetTalents()
	self.resources:GetBuffs()
	self.resources:GetSpellCosts()

	-- Find where Kill Shot KS is placed on the actionbars
	if FS.spec == "BM" or FS.spec == "MM" then
		self:RegisterEvent("ACTIONBAR_SLOT_CHANGED") -- For Kill Shot CD
		self:FindKillShotSlot()
	end
	
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

-- fires when something about your talents changes
function FS:PLAYER_TALENT_UPDATE()
	self.spec = self:GetTalents()
	self.resources:GetBuffs()
	self.resources:GetSpellCosts()
	
	if self.spec == "BM" or self.spec == "MM" then
		self:RegisterEvent("ACTIONBAR_SLOT_CHANGED") -- For Kill Shot CD
		self:FindKillShotSlot()
		--if self.ksslot == 0 then FS:Print("Place Kill Shot onto action bars for FaceShooter to work properly") end
	else
		self:UnregisterEvent("ACTIONBAR_SLOT_CHANGED")
	end
end

-- fires when you leave combat
function FS:PLAYER_REGEN_ENABLED()
	self.ooc = true
	self.resources:PurgeDebuffs()
end

-- fires when you enter combat
function FS:PLAYER_REGEN_DISABLED()
	self.ooc = false
end

-- fires when someone leaves/enters or is moved in your party/raid
function FS:PARTY_MEMBERS_CHANGED()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.GetNumPlayers then module:GetNumPlayers() end
	end
	if self:IsLegitTarget() then
		self:AcquireTarget()
	else
		self:LooseTarget()
	end
end

-- fires when you get/loose/change your target
function FS:PLAYER_TARGET_CHANGED()
	if self:IsLegitTarget() then
		self:AcquireTarget()
	else
		self:LooseTarget()
	end
end

-- fires when your targets faction changes (e.g. becomes attackable)
function FS:UNIT_FACTION()
	if self:IsLegitTarget() then
		self:AcquireTarget()
	else
		self:LooseTarget()
	end
end

-- fires whenever a buff on you or someone else is gained or drops off
function FS:UNIT_AURA(event, unitID)
	-- check for your own buffs
	if unitID == "player" then
		self.resources:GetBuffs()
	-- check for targets debuffs
	elseif unitID == "target" then
		self.resources:GetDebuffs()
	end
end

function FS:ACTIONBAR_SLOT_CHANGED(_, slot)
	FS:FindKillShotSlot(slot)
end

function FS:ACTIONBAR_UPDATE_COOLDOWN(btn)
	local start, duration, enable = GetActionCooldown(FS.ksslot)
	--FS:Print("Start:"..start.." Dur:"..duration.." Enable:"..enable)
end

-- fires whenever the player sends a spellcast to the server
function FS:UNIT_SPELLCAST_SENT(event, unitID, spell)
	local now = GetTime()
	
	if unitID == "player" and
		(spell == GetSpellInfo(FS.spellID["SS"]) or
		spell == GetSpellInfo(FS.spellID["CoS"]) or
		spell == GetSpellInfo(FS.spellID["CS"]) or
		spell == GetSpellInfo(FS.spellID["BA"]) or
		spell == GetSpellInfo(FS.spellID["ES"]) or
		spell == GetSpellInfo(FS.spellID["AS"]) or
		spell == GetSpellInfo(FS.spellID["KS"]) or
		spell == GetSpellInfo(FS.spellID["KC"]) or
		spell == GetSpellInfo(FS.spellID["ET"]) or
		spell == GetSpellInfo(FS.spellID["DB"]) or
		spell == GetSpellInfo(FS.spellID["AMoC"]) or
		spell == GetSpellInfo(FS.spellID["GT"]) or
		spell == GetSpellInfo(FS.spellID["PS"]) or
		spell == GetSpellInfo(FS.spellID["BRG"]) or
		spell == GetSpellInfo(FS.spellID["FS"]) or
		spell == GetSpellInfo(FS.spellID["AimS"]))
	then
		local steady, cobra, focusing = GetSpellInfo(self.spellID["SS"]), GetSpellInfo(self.spellID["CoS"]), GetSpellInfo(self.spellID["FS"])
		self.lastCastSentAt = now
		for key in pairs(self.resources.lastCast) do
			if spell == GetSpellInfo(self.spellID[key]) then self.resources.lastCast[key] = now end
		end
		
		-- if spellName[1] is set put spell on the queue until current spell finishes casting
		if self.spellName[1] and self.gcd < now then
			self.spellName[0] = spell
			self.spellSentAt[0] = now
			--self.spellStartedAt[0] = self.castEndsAt
			self.spellEndedAt[0] = now + self:GetCastDelta(spell, self.resources.haste)
			if self.spellName[1] == steady and spell == steady then self.resources.focusGains["SS"] = 2 end
			if self.spellName[1] == cobra and spell == cobra then self.resources.focusGains["CoS"] = 2 end
			if self.spellName[1] == focusing and spell == focusing then self.resources.focusGains["FS"] = 2 end
			if self.spellName[1] == steady or spell == steady then self.resources.focusGains["SS"] = 1 end
			if self.spellName[1] == cobra or spell == cobra then self.resources.focusGains["CoS"] = 1 end
			if self.spellName[1] == focusing or spell == focusing then self.resources.focusGains["FS"] = 1 end
			-- otherwise casting of the new spell commences
		elseif not self.spellName[1] then
			self.state = "castSent"
			self.spellName[1] = spell
			self.spellSentAt[1] = now
			self.spellStartedAt[1] = nil
			self.spellEndedAt[1] = now + self:GetCastDelta(spell, self.resources.haste)
			self.gcd = now + 1 --+ self.avrgLatency
			if self.spellName[2] ~= steady then self.resources.focusGains["SS"] = 0 end
			if self.spellName[2] ~= cobra then self.resources.focusGains["CoS"] = 0 end
			if self.spellName[2] ~= focusing then self.resources.focusGains["FS"] = 0 end
			if self.spellName[2] == steady and self.resources.focusGains["SS"] > 1 then self.resources.focusGains["SS"] = 1 end
			if self.spellName[2] == cobra and self.resources.focusGains["CoS"] > 1 then self.resources.focusGains["CoS"] = 1 end
			if self.spellName[2] == focusing and self.resources.focusGains["FS"] > 1 then self.resources.focusGains["FS"] = 1 end
			if spell == steady then self.resources.focusGains["SS"] = self.resources.focusGains["SS"] + 1 end
			if spell == cobra then self.resources.focusGains["CoS"] = self.resources.focusGains["CoS"] + 1 end
			if spell == focusing then self.resources.focusGains["FS"] = self.resources.focusGains["FS"] + 1 end
			self:Scheduler()
		end
	end
	
	--Draw CDF only if an instant shot
	if unitID == "player" and
		(spell == GetSpellInfo(FS.spellID["CS"]) or
		spell == GetSpellInfo(FS.spellID["BA"]) or
		spell == GetSpellInfo(FS.spellID["ES"]) or
		spell == GetSpellInfo(FS.spellID["AS"]) or
		spell == GetSpellInfo(FS.spellID["KS"]) or
		spell == GetSpellInfo(FS.spellID["KC"]) or
		spell == GetSpellInfo(FS.spellID["ET"]) or
		spell == GetSpellInfo(FS.spellID["DB"]) or
		spell == GetSpellInfo(FS.spellID["AMoC"]) or
		spell == GetSpellInfo(FS.spellID["GT"])) then
		
		
		--FS:Print("Spell_Cast_Sent DrawCDF")
		self.button1:DrawCDF(self.spellSentAt[1] , self.spellEndedAt[1] - self.spellSentAt[1])
	end
	
end

-- fires when spellcast has been acknowledged by the server
function FS:UNIT_SPELLCAST_START(event, unitID, spell, _, num)
--	FS:Print("Spell Start:"..spell)
	if (unitID == "player") and
		(spell == GetSpellInfo(FS.spellID["SS"]) or
		spell == GetSpellInfo(FS.spellID["CoS"]) or
		spell == GetSpellInfo(FS.spellID["AimS"]) or
		spell == GetSpellInfo(FS.spellID["FS"]) or
		spell == GetSpellInfo(FS.spellID["PS"]))
	then
		local now = GetTime()
		self.lastCastStartedAt = now
		self.state = "castStarted"
		self.spellNum[num] = spell
		self.spellName[0] = nil
		self.spellSentAt[0] = nil
		self.spellStartedAt[0] = nil
		self.spellEndedAt[0] = nil
		self.spellName[1] = spell
		self.spellStartedAt[1] = now
		-- in some cases wow seems to allow to sent casts before gcd is up and the spellcast is wrongly ignored
		if not self.spellSentAt[1] then
			self.spellSentAt[1] = now - self.avrgLatency
			self.spellEndedAt[1] = self.spellSentAt[1] + self:GetCastDelta(spell, self.resources.haste)
		end
		if self.spellSentAt[1] then self.avrgLatency = self:GetAverage("Latency", (now - self.spellSentAt[1])) end
	end
	
		--Draw CDF only if a cast shot
	if self.spellEndedAt[1] ~= nil and self.spellStartedAt[1] ~= nil then
		--FS:Print("Spell_Cast_Start DrawCDF")
		self.button1:DrawCDF(self.spellSentAt[1] , self.spellEndedAt[1] - self.spellSentAt[1])
	end
end

-- fires when spellcast has succeeded
function FS:UNIT_SPELLCAST_SUCCEEDED(event, unitID, spell, _, num)
--	if spell ~=  GetSpellInfo(FS.spellID["AutoS"]) then
--		FS:Print(spell.." succeeded")
--	end
	
	if unitID == "player" then
		if (spell == GetSpellInfo(FS.spellID["SS"]) or
			spell == GetSpellInfo(FS.spellID["CoS"]) or
			spell == GetSpellInfo(FS.spellID["CS"]) or
			spell == GetSpellInfo(FS.spellID["BA"]) or
			spell == GetSpellInfo(FS.spellID["ES"]) or
			spell == GetSpellInfo(FS.spellID["AS"]) or
			spell == GetSpellInfo(FS.spellID["KS"]) or
			spell == GetSpellInfo(FS.spellID["KC"]) or
			spell == GetSpellInfo(FS.spellID["ET"]) or
			spell == GetSpellInfo(FS.spellID["DB"]) or
			spell == GetSpellInfo(FS.spellID["AMoC"]) or
			spell == GetSpellInfo(FS.spellID["GT"]) or
			spell == GetSpellInfo(FS.spellID["PS"]) or
			spell == GetSpellInfo(FS.spellID["FS"]) or
			spell == GetSpellInfo(FS.spellID["AimS"]))
		then
			local now = GetTime()
			self.lastCastSucceededAt = now
			if self.spellNum[num] then self.spellNum[num] = nil end

			if not self.spellName[0] then
				table.insert(self.spellName, 1, nil)
				table.remove(self.spellName, 5)
				table.insert(self.spellSentAt, 1, nil)
				table.remove(self.spellSentAt, 5)
				table.insert(self.spellStartedAt, 1, nil)
				table.remove(self.spellStartedAt, 5)
				table.insert(self.spellEndedAt, 1, nil)
				table.remove(self.spellEndedAt, 5)
				if (spell == GetSpellInfo(FS.spellID["SS"])
					or spell == GetSpellInfo(FS.spellID["CoS"])
					or spell == GetSpellInfo(FS.spellID["AimS"])
					or spell == GetSpellInfo(FS.spellID["PS"])
					or spell == GetSpellInfo(FS.spellID["FS"])
					or spell == GetSpellInfo(FS.spellID["BRG"]))
				then
					self.state = "waiting"
				elseif self.gcd >= now then
					self.state = "succeeded"
					self.spellStartedAt[2] = now
					self.avrgLatency = self:GetAverage("Latency", (now - self.lastCastSentAt))
				end
			else
				self.state = "castSent"
				table.insert(self.spellName, 1, self.spellName[0])
				table.remove(self.spellName, 5)
				table.insert(self.spellSentAt, 1, self.spellSentAt[0])
				table.remove(self.spellSentAt, 5)
				table.insert(self.spellStartedAt, 1, now)
				table.remove(self.spellStartedAt, 5)
				table.insert(self.spellEndedAt, 1, self.spellEndedAt[0])
				table.remove(self.spellEndedAt, 5)
				self.spellName[0] = nil
				self.spellSentAt[0] = nil
				self.spellStartedAt[0] = nil
				self.spellEndedAt[0] = nil
				--self.gcd = now + 1
				self:Scheduler()
			end
		elseif spell == GetSpellInfo(FS.spellID["BRG"]) then
			--FS:Print("Spell Succeeded: BRG")
			local name, _, _, _, startTime, endTime = UnitChannelInfo("player")
			--if name then
			--	FS:Print("BRG: Start:"..FS:Round(startTime,2).." Dur:"..FS:Round(endTime-startTime,3))
			--end
			
			if name == "Barrage" and startTime ~= nil and endTime ~= nil then -- Barrage is channelling
				startTime = startTime / 1000
				endTime = endTime / 1000
				if self.spellEndedAt[1] ~= endTime then --This is our first time through
					local now = GetTime()
					--FS:Print(now)
					self.lastCastStartedAt = startTime
					self.state = "castStarted"
					self.spellNum[num] = spell
					self.spellName[0] = nil
					self.spellSentAt[0] = nil
					self.spellStartedAt[0] = nil
					self.spellEndedAt[0] = nil
					self.spellName[1] = spell
					self.spellStartedAt[1] = startTime
					self.spellEndedAt[1] = endTime
					self.button1:DrawCDF(startTime, endTime - startTime)
				else
				--Something else 
				end
			else -- Our cast is not barrage and has finished
				table.insert(self.spellName, 1, nil)
				table.remove(self.spellName, 5)
				table.insert(self.spellSentAt, 1, nil)
				table.remove(self.spellSentAt, 5)
				table.insert(self.spellStartedAt, 1, nil)
				table.remove(self.spellStartedAt, 5)
				table.insert(self.spellEndedAt, 1, nil)
				table.remove(self.spellEndedAt, 5)
				self.state = "waiting"
			end
		end	
	end
end

-- when the GCD starts or ends
function FS:SPELL_UPDATE_COOLDOWN()
	
	local now = GetTime()
	local oldGCD = self.gcd

	-- if off gcd and still in latency state after some grace period return to waiting state
	if self.gcd < now and self.state == "latency" and (self.lastCastSucceededAt + 0.1) < now then
		-- if not off gcd according to last saved gcd timestamp assume cast has not been sucessful and gcd has been reset
		if oldGCD > now then
			if self.spellName[1] == GetSpellInfo(self.spellID["ES"]) then
				self.resources.es = 0
			elseif self.spellName[1] == GetSpellInfo(self.spellID["KC"]) then
				--self.resources.focusGains["RiF"] = 0
			end
			self.spellName[1] = nil
			self.spellSentAt[1] = nil
			self.spellStartedAt[1] = nil
			self.spellEndedAt[1] = nil
		end
		self.state = "waiting"
	end
	
	local start,duration = FS:GetGCD()
	--FS:Print("SUD: Start:"..start.." Duartion:"..duration)
	--if start ~= 0 and duration ~= 0 and FS.state ~= "castStarted" then
	--	FS:Print("Spell_Update Cooldown DrawCDF")
	--	self.button1:DrawCDF(start,duration)
	--end
end

-- pick up energizing effects
function FS:COMBAT_LOG_EVENT_UNFILTERED(event, timeStamp, combatEvent, _, srcGUID, srcName, srcFlags, _, dstGUID, dstName, dstFlags, _, spellID, spellName, spellSchool, amount, powerType)
	if srcGUID == self.guid then
		--FS:Print("Name:"..spellName.." Combat Event:"..combatEvent)
	end

	if srcGUID == self.guid and combatEvent == "SPELL_ENERGIZE" then
		-- Steady Shot
		if spellID == 77443 then
			if self.spellName[0] == GetSpellInfo(self.spellID["SS"]) then self.resources.focusGains["SS"] = 1
			else self.resources.focusGains["SS"] = 0
			end
		-- Cobra Shot
		elseif spellID == 91954 then
			if self.spellName[0] == GetSpellInfo(self.spellID["CoS"]) then self.resources.focusGains["CoS"] = 1
			else self.resources.focusGains["CoS"] = 0
			end
		end
	end
end
