local FS = LibStub("AceAddon-3.0"):GetAddon("FaceShooter")

-- setup slash commmands
function FS:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
		LibStub("AceConfigCmd-3.0").HandleCommand(FS, "fs", "FaceShooter", input)
    end
end

function FS:IsDebug(info)
	return self.debug
end

function FS:ToggleDebug(info, value)
	if value then
		self:Print("Debug mode activated.")
		if self.debugFrm then self.debugFrm.btn:Show()
		else self.debugFrm = self:CreateDebugFrame()
		end
	else
		self:Print("Debug mode deactivated.")
		self.debugFrm.btn:Hide()
	end
	self.debug = value
end

function FS:IsLog(info)
	return self.log
end

function FS:ToggleLog(info, value)
	if value then
		self:Print("Log mode activated.")
		self.db.profile.log = {}
	else
		self:Print("Log mode deactivated.")
		self.db.profile.log = nil
	end
	self.log = value
end

function FS:IsLock(info)
	return self.db.profile.lock
end

function FS:ToggleLock(info, value)
	local i
	if value then
		for i = 1, 2 do
			if self.db.profile["button"..i]["enabled"] then
				self["button"..i]["mov"]:EnableMouse(false)
				self["button"..i]["mov"]:SetAlpha(0)
			end
		end
		for name, module in FS:IterateModules() do
			if module:IsEnabled() and module.Lock then module:Lock() end
		end
		self:Print("Frames locked.")
	else
		for i = 1, 2 do
			if self.db.profile["button"..i]["enabled"] then
				self["button"..i]["mov"]:EnableMouse(true)
				self["button"..i]["mov"]:SetAlpha(1)
			end
		end
		for name, module in FS:IterateModules() do
			if module:IsEnabled() and module.UnLock then module:UnLock() end
		end
		self:Print("Frames unlocked.")
	end
	self.db.profile.lock = value
end

function FS:IsShowInRaid(info)
	return self.db.profile.showInRaid
end

function FS:ToggleShowInRaid(info, value)
	local usedToBeLegitTarget = self:IsLegitTarget()
	self.db.profile.showInRaid = value
	if value then self:Print("Frames are shown when in raid now.")
	else self:Print("Frames are no longer shown when in raid now.")
	end
	if self:IsLegitTarget() and not usedToBeLegitTarget then self:AcquireTarget()
	elseif not self:IsLegitTarget() and usedToBeLegitTarget then self:LooseTarget()
	end
end

function FS:IsShowInParty(info)
	return self.db.profile.showInParty
end

function FS:ToggleShowInParty(info, value)
	local usedToBeLegitTarget = self:IsLegitTarget()
	self.db.profile.showInParty = value
	if value then self:Print("Frames are shown when in party now.")
	else self:Print("Frames are no longer shown when in party now.")
	end
	if self:IsLegitTarget() and not usedToBeLegitTarget then self:AcquireTarget()
	elseif not self:IsLegitTarget() and usedToBeLegitTarget then self:LooseTarget()
	end
end

function FS:IsShowWhenSolo(info)
	return self.db.profile.showWhenSolo
end

function FS:ToggleShowWhenSolo(info, value)
	local usedToBeLegitTarget = self:IsLegitTarget()
	self.db.profile.showWhenSolo = value
	if value then self:Print("Frames are shown when solo now.")
	else self:Print("Frames are no longer shown when solo now.")
	end
	if self:IsLegitTarget() and not usedToBeLegitTarget then self:AcquireTarget()
	elseif not self:IsLegitTarget() and usedToBeLegitTarget then self:LooseTarget()
	end
end

function FS:GetMaxWait(info)
	return self.db.profile.maxWait
end

function FS:SetMaxWait(info, value)
	self.db.profile.maxWait = value
	if info.uiType == "cmd" then self:Print("Maximum time to wait for signature shot to cooldown set to "..value.."ms.") end
end

function FS:GetSuggestBA(info)
	return self.db.profile.ba.suggestBA
end

function FS:SetSuggestBA(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.ba.suggestBA then
		if value == "never" then self:Print("Never suggesting Black Arrow in rotations.")
		elseif value == "level" then self:Print("Only suggest Black Arrow when target has bosslevel.")
		else self:Print("Always suggest Black Arrow.")
		end
	end
	self.db.profile.ba.suggestBA = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end


function FS:GetETTTL(info)
	return self.db.profile.ET.minTTL
end

function FS:SetETTTL(info,value)
	self.db.profile.ET.minTTL = value
	if info.uiType == "cmd" then self:Print("Minimum TTL to suggest using Explosive Trap set to "..value.."sec.") end
end

function FS:GetDbTTL(info)
	return self.db.profile.db.minTTL
end

function FS:SetDbTTL(info,value)
	self.db.profile.db.minTTL = value
	if info.uiType == "cmd" then self:Print("Minimum TTL to suggest using Dire Beast set to "..value.."sec.") end
end

function FS:GetSuggestDB(info)
	return self.db.profile.db.suggestDB
end

function FS:SetSuggestDB(info, value)
	--self.Print("Current Dire Beast Option: "..self.db.profile.db.suggestDB)
	--self.Print("New Dire Beast Option: "..value)
	if info.uiType == "cmd" and value ~= self.db.profile.db.suggestDB then
		if value == "never" then self:Print("Never suggesting Dire Beast in rotations.")
		elseif value == "level" then self:Print("Only suggest Dire Beast when target is Boss Level.")
		else self:Print("Always suggest Dire Beast.")
		end
	end
	self.db.profile.db.suggestDB = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:GetSuggestAMoC(info)
	return self.db.profile.AMoC.suggestAMoC
end

function FS:SetSuggestAMoC(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.AMoC.suggestAMoC then
		if value == "never" then self:Print("Never suggesting A Murder of Crows in rotations.")
		elseif value == "level" then self:Print("Only suggest A Murder of Crows when target is Boss Level.")
		else self:Print("Always suggest A Murder of Crows.")
		end
	end
	self.db.profile.AMoC.suggestAMoC = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:GetAMoCTTL(info)
	return self.db.profile.AMoC.minTTL
end

function FS:SetAMoCTTL(info,value)
	self.db.profile.AMoC.minTTL = value
	if info.uiType == "cmd" then self:Print("Minimum TTL to suggest using A Murder of Crows set to "..value.."sec.") end
end

function FS:GetSuggestGT(info)
	return self.db.profile.GT.suggestGT
end

function FS:SetSuggestGT(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.gt.suggestGT then
		if value == "never" then self:Print("Never suggesting Glaive Toss in rotations.")
		elseif value == "level" then self:Print("Only suggest Glaive Toss when target is Boss Level.")
		else self:Print("Always suggest Glaive Toss.")
		end
	end
	self.db.profile.GT.suggestGT = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:GetSuggestCA(info)
	return self.db.profile.CA.suggestCA
end

function FS:SetSuggestCA(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.gt.suggestGT then
		if value == "never" then self:Print("Never prioritize Aimed and Steady shots when Careful aim is active.")
		elseif value == "level" then self:Print("Only prioritize when target is Boss Level.")
		else self:Print("Always prioritize Aimed and Steady shots when Careful aim is active.")
		end
	end
	self.db.profile.CA.suggestCA = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:GetSuggestPS(info)
	return self.db.profile.PS.suggestPS
end


function FS:SetSuggestPS(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.PS.suggestPS then
		if value == "never" then self:Print("Never suggesting Power Shot in rotations.")
		elseif value == "level" then self:Print("Only suggest Power Shot when target is Boss Level.")
		else self:Print("Always suggest Power Shot.")
		end
	end
	self.db.profile.PS.suggestPS = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:GetSuggestBrg(info)
	return self.db.profile.Brg.suggestBrg
end

function FS:SetSuggestBrg(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.Brg.suggestBrg then
		if value == "never" then self:Print("Never suggesting Barrage in rotations.")
		elseif value == "level" then self:Print("Only suggest Barrage when target is Boss Level.")
		else self:Print("Always suggest Barrage.")
		end
	end
	self.db.profile.Brg.suggestBrg = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:GetSuggestET(info)
	return self.db.profile.ET.suggestET
end

function FS:SetSuggestET(info, value)
	if info.uiType == "cmd" and value ~= self.db.profile.ET.suggestET then
		if value == "never" then self:Print("Never suggesting Explosive Trap in rotations.")
		elseif value == "level" then self:Print("Only suggest Explosive Trap when target has bosslevel.")
		else self:Print("Always suggest Explosive Trap.")
		end
	end
	self.db.profile.ET.suggestET = value
	self:CheckForDisabled()
	for name, module in FS:IterateModules() do
		if module:IsEnabled() and module.CheckForDisabled then module:CheckForDisabled() end
	end
end

function FS:IsSaveFocusBA(info)
	return self.db.profile.ba.saveFocusBA
end

function FS:ToggleSaveFocusBA(info, value)
	self.db.profile.ba.saveFocusBA = value
	if value then self:Print("Saving focus for signature shot when suggesting Black Arrow now.")
	else self:Print("No longer saving focus for signature shot when suggesting Black Arrow now.")
	end
end

function FS:IsEnabled(info)
	return self.db.profile[(info[#info-1])]["enabled"]
end

function FS:ToggleEnabled(info, value)
	local name = (info[#info-1])
	local i
	if value then
		if name == "button1" or name == "button2" then
			self:Print("Enabled second recommendation frame.")
			self.button2 = self.button2 or self:CreateButton("button2")
			self.db.profile.button2.enabled = true
			self.db.profile.button2.wasEnabled = true
		else
			self:Print(self.modNames[name].." module enabled.")
			self.db.profile[name]["enabled"] = true
			FS:EnableModule(self.modNames[name])
		end
	else
		if name == "button1" or name == "button2" then
			self:Print("Disabled second recommendation frame.")
			if self.button2 then self:RemoveButton("button2") end
			self.db.profile.button2.enabled = false
			self.db.profile.button2.wasEnabled = false
		else
			self:Print(self.modNames[name].." module disabled.")
			self.db.profile[name]["enabled"] = false
			FS:DisableModule(self.modNames[name])
		end
	end
end

function FS:GetScale(info)
	return self.db.profile[(info[#info-1])]["scale"]
end

function FS:SetScale(info, value)
	local name = info[#info-1]
	if name == "button1" or name == "button2" then
		self[name]["mov"]:SetScale(value/100)
		self[name]["btn"]:SetScale(value/100)
	else
		local module = FS:GetModule(self.modNames[name])
		module.mov:SetScale(value/100)
	end
	self.db.profile[(info[#info-1])]["scale"] = value
	if info.uiType == "cmd" then self:Print("Scale set to "..value.."%") end
end

function FS:GetAlpha(info)
	return self.db.profile[(info[#info-1])]["alpha"]
end

function FS:SetAlpha(info, value)
	local name = info[#info-1]
	if name == "button1" or name == "button2" then
		self[(info[#info-1])]["btn"]:SetAlpha(value/100)
	else
		local module = FS:GetModule(self.modNames[name])
		module.frm:SetAlpha(value/100)
	end
	self.db.profile[(info[#info-1])]["alpha"] = value
	if info.uiType == "cmd" then self:Print("Alpha set to "..value.."%") end
end
